require 'thor'
require 'faraday'
require 'colorize'
require 'yaml'
require 'json'

module Fantasia

  class CLI < Thor

    class_option :force, type: :boolean, aliases: "-f", desc: "Ignore unstaged git changes"

    desc 'build', 'Build the stack locally'
    def build
      if !system("git diff-index --quiet HEAD --")
        if options[:force]
          STDERR.puts "Ignoring uncommitted changes. This is probably a bad idea.".colorize(:red)
        else
          STDERR.puts "There are uncommitted changes. Commit, stash, or revert them, or use the force option.".colorize(:red)
          exit(1)
        end
      end
      system("docker-compose build") || exit(1)
    end

    desc 'push', 'Push the stack to the registry'
    def push
      system("docker-compose push") || exit(1)
    end

    desc 'update', 'Update the stack in Fantasia'
    def update
      spec = YAML.load(File.read "fantasia.yml")
      secrets = YAML.load(File.read(spec["secrets"] || "fantasia.secrets.yml"))
      url = spec["url"]
      http = Faraday.new("#{url}api/v1/")
      name = spec["stack"]
      http.authorization(:Bearer, secrets[name])
      r = http.post "stack", {
        fantasia_file: spec.to_yaml,
        compose_file: File.read(spec["compose-file"] || "docker-compose.yml")
      }
      r = JSON.parse(r.body)
      if r["success"]
        puts r["message"].colorize(:green)
      else
        puts r["message"].colorize(:red)
        exit 1
      end
    end

    desc 'deploy', 'Build, push, and update the stack'
    def deploy
      invoke :build
      invoke :push
      invoke :update
    end

    desc 'show', 'Show stack information'
    def show
      spec = YAML.load(File.read "fantasia.yml")
      secrets = YAML.load(File.read(spec["secrets"] || "fantasia.secrets.yml"))
      url = spec["url"]
      http = Faraday.new("#{url}api/v1/")
      name = spec["stack"]
      http.authorization(:Bearer, secrets[name])
      r = JSON.parse(http.get("stack").body)
      puts "#{"State".colorize(:green)}:        #{r["state"]}"
      puts "#{"Portainer ID".colorize(:green)}: #{r["portainer_id"]}"
      puts
      puts "Fantasia config".colorize(:green)
      puts r["fantasia_file"]
      puts
      puts "Compose config".colorize(:green)
      puts r["compose_file"]
    end

  end

end
